package com.huigou.topsun.base.bsnMessage.appliction.impl;

import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.base.bsnMessage.appliction.BsnMessageApplication;
import com.huigou.topsun.base.bsnMessage.domain.*;
import com.huigou.topsun.base.bsnMessage.repository.BsnMessageExecuteRepository;
import com.huigou.topsun.base.bsnMessage.repository.BsnMessageRejectRepository;
import com.huigou.topsun.base.bsnMessage.repository.BsnMessageSponsorRepository;
import com.huigou.topsun.sap.nonProdApply.domain.SapNonProdApplyDept;
import com.huigou.uasp.bmp.bizconfig.model.ObjectKindEnum;
import com.huigou.uasp.bmp.common.application.BaseApplication;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Sort;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 专项任务整合及输出
 */
@Service("bsnMessageApplication")
public class BsnMessageApplicationImpl extends BaseApplication implements BsnMessageApplication {

    @Autowired
    BsnMessageExecuteRepository messageExecuteRepository;
    @Autowired
    BsnMessageRejectRepository messageRejectRepository;
    @Autowired
    BsnMessageSponsorRepository messageSponsorRepository;


    @Override
    public String keepTasks(BsnMessageSponsor bsnMessageSponsor) {
        //删除对应表单已有的任务
        BsnMessageSponsor sponsor = this.getSponsorByBId(bsnMessageSponsor.getBusinessId());
        if(sponsor!=null){
            this.deleteBsnMessageSponsor(sponsor);
        }
        //封装任务发布人自己的审批任务
        BsnMessageExecute execute = new BsnMessageExecute();
        execute.setUpdateDate(new Date());
        execute.setTaskExecute(this.getOperator().getUserId());
        execute.setTaskDescribe("各单位已完成[发起人审核]");
        execute.setBusinessCode("system");
        execute.setActive(ActiveStatus.SAVED.getValue());
        bsnMessageSponsor.addExecute(execute);
        return this.saveBsnMessageSponsor(bsnMessageSponsor);
    }

    @Override
    public void publishTasks(String bId) {
        BsnMessageSponsor sponsor = messageSponsorRepository.findByBusinessId(bId);
        if(sponsor==null){
            throw new RuntimeException("不存在该业务");
        }
        List<BsnMessageExecute> executes = this.getExecuteByMsId(sponsor.getId());
        this.updateActiveStatus(executes);
        this.saveBsnMessageExecute(executes);
    }


    @Override
    public void keepAndPublishTasks(BsnMessageSponsor bsnMessageSponsor) {
        this.updateActiveStatus(bsnMessageSponsor.getExecutes());
        this.keepTasks(bsnMessageSponsor);
    }

//    @Override
//    public void authorizeTasks(String bId) {
//        BsnMessageSponsor sponsor = messageSponsorRepository.findByBusinessId(bId);
//    }

//    @Override
//    public void rejectTasks(BsnMessageExecute execute) {
//        //更改驳回状态
//        execute.setActive(ActiveStatus.REJECTED.getValue());
//        //更新时间
//        execute.setUpdateDate(new Date());
//        //封装驳回详细
//        BsnMessageReject reject = new BsnMessageReject();
//        reject.setRejectMsg(sdo.getString(key));
//        reject.setRejectDate(new Date());
//        reject.setBsnExecuteId(execute.getId());
//        execute.addReject(reject);
//        this.saveBsnMessageExecute(execute);
//    }


    @Override
    public Map<String, Object> circulate(String meId) {
        BsnMessageExecute execute = this.getExecuteByMeId(meId);
        Map<String,Object> paramMap = new HashMap<>();
        paramMap.put("deptCode", execute.getBusinessCode());
        paramMap.put("active",execute.getActive());
        paramMap.put("meId",execute.getId());
        paramMap.put("isReadOnly","false");
        //获取该业务流程下其他执行部门
        List<BsnMessageExecute> executeAll = this.getBrotherExecuteByMeId(meId);
        Map<String, String> map = executeAll.stream()
                .collect(Collectors.toMap(BsnMessageExecute::getTaskExecute, BsnMessageExecute::getBusinessCode));

        paramMap.put("auditDeptMap", map);
        return paramMap;
    }

    @Override
    public Map<String, Object> queryExecuteBsnMessage() {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "queryExecuteBsnMessage");
        QueryModel queryModel = new QueryModel();
        queryModel.setSql(queryDescriptor.getSql());
        queryModel.setPageIndex(1);
        queryModel.setPageSize(10);
        queryModel.setSortFieldName("updateDate");
        queryModel.setSortOrder("desc");
        queryModel.putParam("taskExecute",this.getOperator().getUserId());
        queryModel.putParam("addText", "[驳回]");
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public List<BsnMessageSponsor> queryFinishBsnMessageSponsor() {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "queryFinishBsnMessageSponsor");
        return this.sqlExecutorDao.queryToList(queryDescriptor.getSql(), BsnMessageSponsor.class,this.getOperator().getUserId());
    }


    @Override
    public BsnMessageExecute isExecutesReady(String msId) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "queryIsExecutesReady");
        return this.sqlExecutorDao.queryToObject(queryDescriptor.getSql(),BsnMessageExecute.class,msId,msId);
    }

    public void saveBsnMessageSponsor(List<BsnMessageSponsor> all){
        messageSponsorRepository.save(all);
    }

    public String saveBsnMessageSponsor(BsnMessageSponsor entity){
        String keyId = messageSponsorRepository.save(entity).getId();
        if(isNotEmpty(entity.getExecutes())){
            entity.getExecutes().forEach(item->item.setMessageSponsorId(keyId));
            messageExecuteRepository.save(entity.getExecutes());
        }
        return keyId;
    }


    public void saveBsnMessageExecute(List<BsnMessageExecute> all){
        messageExecuteRepository.save(all);
    }
    public String saveBsnMessageExecute(BsnMessageExecute entity){
        String keyId = messageExecuteRepository.save(entity).getId();
        if(isNotEmpty(entity.getRejects())){
            entity.getRejects().forEach(item->item.setBsnExecuteId(keyId));
            messageRejectRepository.save(entity.getRejects());
        }
        return keyId;
    }
    public void saveBsnMessageReject(List<BsnMessageReject> all){
        messageRejectRepository.save(all);
    }
    public void saveBsnMessageReject(BsnMessageReject entity){
        messageRejectRepository.save(entity);
    }

    @Override
    public void deleteBsnMessageSponsor(BsnMessageSponsor entity) {
        if(isNotEmpty(entity.getExecutes())){
            messageExecuteRepository.delete(entity.getExecutes());
        }
        messageSponsorRepository.delete(entity);
    }

    @Override
    public List<BsnMessageExecute> getExecuteByMsId(String msId) {
        return messageExecuteRepository.findAllByMessageSponsorId(msId);
    }

    @Override
    public BsnMessageExecute getExecuteByMeId(String id) {
        return messageExecuteRepository.findOne(id);
    }

    @Override
    public List<BsnMessageReject> getRejectByMeId(String meId) {
        Sort sort = new Sort(Sort.Direction.DESC, "rejectDate");
        return messageRejectRepository.findAllByBsnExecuteId(meId,sort);
    }

    @Override
    public List<BsnMessageExecute> getBrotherExecuteByMeId(String meId) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "queryBrotherExecute");
        Map<String,Object> mapParam = new HashMap<>();
        mapParam.put("meId",meId);
        return this.sqlExecutorDao.queryToListByMapParam(queryDescriptor.getSql(), BsnMessageExecute.class, mapParam);
    }

    @Override
    public BsnMessageSponsor getSponsorByBId(String bId) {
        BsnMessageSponsor sponsor = messageSponsorRepository.findByBusinessId(bId);
        if(sponsor!=null){
            List<BsnMessageExecute> executeList =this.getExecuteByMsId(sponsor.getId());
            sponsor.setExecutes(executeList);
        }
        return sponsor;
    }

    public boolean isNotEmpty(List<?> list) {
        return list != null && !list.isEmpty();
    }


    public void updateActiveStatus( List<BsnMessageExecute> executes) {
        //将除审批任务外状态为保存未分发的全部修改为分发未完成
        for (BsnMessageExecute execute : executes) {
            if(!"system".equals(execute.getBusinessCode())){
                execute.setActive(ActiveStatus.UNFINISHED.getValue());
            }
        }
    }
}
