package com.huigou.topsun.sap.nonProdApply.application.impl;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.huigou.topsun.base.bsnMessage.appliction.BsnMessageApplication;
import com.huigou.topsun.base.bsnMessage.domain.ActiveStatus;
import com.huigou.topsun.base.bsnMessage.domain.BsnMessageExecute;
import com.huigou.topsun.base.bsnMessage.domain.BsnMessageReject;
import com.huigou.topsun.base.bsnMessage.domain.BsnMessageSponsor;
import com.huigou.topsun.sap.nonProdApply.application.NonProdApplyDeptApplication;

import com.huigou.topsun.sap.nonProdApply.domain.SapNonProdApplyDept;
import com.huigou.topsun.sap.nonProdApply.repository.NonProdApplyDeptRepository;
import com.huigou.uasp.bmp.common.application.BaseApplication;

import com.huigou.util.SDO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class NonProdApplyDeptApplicationImpl extends BaseApplication implements NonProdApplyDeptApplication {

    @Autowired
    NonProdApplyDeptRepository nonProdApplyDeptRepository;
    @Autowired
    BsnMessageApplication bsnMessageApplication;

    @Override
    public Map<String,Object> saveSapNonProdApplyDept(SDO sdo){
        Map<String,Object> map = new HashMap<>();
        SapNonProdApplyDept nonProdApplyDept = sdo.toObject(SapNonProdApplyDept.class);
        nonProdApplyDept = nonProdApplyDeptRepository.save(nonProdApplyDept);

        //获取分发的部门与人员信息
        ObjectMapper objectMapper = new ObjectMapper();
        String deptStaffJSON = sdo.getString("deptStaff");
        List<Map<String, String>> deptStaffList = null;
        try {
            deptStaffList = objectMapper.readValue(deptStaffJSON, new TypeReference<List<Map<String, String>>>() {});
        } catch (JsonProcessingException e) {
            throw new RuntimeException("JSON转换失败："+e);
        }


        //封装分发数据
        BsnMessageSponsor bsnMessageSponsor = new BsnMessageSponsor();
        bsnMessageSponsor.setTaskSponsor(this.getOperator().getUserId());
        bsnMessageSponsor.setTitle("非产品/非本成品类物料");
        bsnMessageSponsor.setExecutorUrl("sapNonProdApplyNext/fillInNonProdApplyDept.do");
        bsnMessageSponsor.setCreateDate(new Date());
        bsnMessageSponsor.setBusinessId(nonProdApplyDept.getApplyDeptId());
        bsnMessageSponsor.setStatus(0);

        for (Map<String, String> stringMap : deptStaffList) {
            BsnMessageExecute messageExecute = new BsnMessageExecute();
            String userCode = stringMap.get("userCode");
            String deptName = stringMap.get("deptName");
            String deptCode = stringMap.get("deptCode");
            String[] split = userCode.split("@");
            messageExecute.setTaskExecute(split[0]);
            messageExecute.setBusinessCode(deptCode);
            messageExecute.setActive(ActiveStatus.SAVED.getValue());
            messageExecute.setUpdateDate(new Date());
            messageExecute.setTaskDescribe(deptName+"部门数据维护");
            bsnMessageSponsor.addExecute(messageExecute);
        }
        //保存任务
        bsnMessageApplication.keepTasks(bsnMessageSponsor);
        map.put("applyDeptId",nonProdApplyDept.getApplyDeptId());
        return map;
    }

    /**
     * 发布任务
     * @param sdo
     */
    @Override
    public void distributeSapNonProdApplyDept(SDO sdo) {
        //将状态为保存未分发的全部修改为分发未完成
        Map<String, Object> map = this.saveSapNonProdApplyDept(sdo);
        bsnMessageApplication.publishTasks((String) map.get("applyDeptId"));
    }

    @Override
    public SapNonProdApplyDept getNonProdApplyDeptById(String id) {
        return nonProdApplyDeptRepository.findOne(id);
    }


    @Override
    public void rejectNonProdApplyDept(SDO sdo) {
        String meId = sdo.getString("meId");
        //需要被驳回的部门
        List<BsnMessageExecute> brotherExecuteAll = bsnMessageApplication.getBrotherExecuteByMeId(meId);
        Map<String, BsnMessageExecute> executeMap = new HashMap<>();
        for (BsnMessageExecute execute : brotherExecuteAll) {
            executeMap.put(execute.getTaskExecute(), execute);
        }
        // 遍历 SDO 对象中的 key
        for (String key : sdo.getProperties().keySet()) {
            // 如果存在对应的 BsnMessageExecute，进行操作
            BsnMessageExecute execute = executeMap.get(key);
            if (execute != null) {
                //更改驳回状态
                execute.setActive(ActiveStatus.REJECTED.getValue());
                //更新时间
                execute.setUpdateDate(new Date());
                //封装驳回详细
                BsnMessageReject reject = new BsnMessageReject();
                reject.setRejectMsg(sdo.getString(key));
                reject.setRejectDate(new Date());
                reject.setBsnExecuteId(execute.getId());
                execute.addReject(reject);
                bsnMessageApplication.saveBsnMessageExecute(execute);
            }
        }
        //当前审核任务回退
        BsnMessageExecute systemExecute = bsnMessageApplication.getExecuteByMeId(meId);
        systemExecute.setActive(ActiveStatus.SAVED.getValue());
        bsnMessageApplication.saveBsnMessageExecute(systemExecute);
    }
}
